import QtQuick 2.4

Item {
    id: root
    width: 1440
    height: 810
    focus: true

    Item {
        id: scene
        anchors.fill: parent

        // Hard-coded horse.jpg image size.
        property variant imageSize: Qt.size(1920, 1080)

        ShaderEffect {
            id: image
            property variant src: Image { source: "horse.jpg" }
            property variant textureMatrix: Qt.matrix4x4(1.0, 0.0, 0.0, 0.0,
                                                         0.0, 1.0, 0.0, 0.0,
                                                         0.0, 0.0, 1.0, 0.0,
                                                         0.0, 0.0, 0.0, 1.0)

            vertexShader: "
                uniform highp mat4 qt_Matrix;
                attribute highp vec4 qt_Vertex;
                attribute highp vec2 qt_MultiTexCoord0;
                varying highp vec2 coord;
                void main() {
                    coord = qt_MultiTexCoord0;
                    gl_Position = qt_Matrix * qt_Vertex;
                }"
            fragmentShader: "
                uniform highp mat4 textureMatrix;
                varying highp vec2 coord;
                uniform sampler2D src;
                uniform lowp float qt_Opacity;
                void main() {
                    highp vec4 transformedCoord = vec4(coord, 0.0, 1.0) * textureMatrix;
                    gl_FragColor = texture2D(src, transformedCoord.st) * qt_Opacity;
                }"

            Connections {
                target: StackFrame
                onFrameChanged: {
                    stackPointer.y = textStack.y + position * 17;
                    image.textureMatrix = textureMatrix;
                    scene.imageSize = imageSize;
                    updateSize();
                }
            }
        }

        Column {
            id: textStack
            anchors.top: parent.top
            anchors.left: parent.left
            anchors.topMargin: 15
            anchors.leftMargin: 25
            Repeater {
                model: [ "Identity", "Rotate 90° clockwise", "Crop 700x1200 @ 200,300",
                         "Rotate 180° clockwise", "Crop 500x350 @ 525,100",
                         "Crop 128x128 @ 0,0", "Rotate 270° clockwise" ]
                Text {
                    font.family: "Ubuntu Mono"
                    font.pixelSize: 16
                    font.weight: Font.Bold
                    style: Text.Outline
                    color: "white"
                    styleColor: "black"
                    text: modelData
                }
            }
        }

        Text {
            id: stackPointer
            x: textStack.anchors.leftMargin - 20
            y: textStack.anchors.topMargin
            font.family: "Ubuntu Mono"
            font.pixelSize: 16
            font.weight: Font.Bold
            style: Text.Outline
            color: "white"
            styleColor: "black"
            text: "→"
        }
    }

    ZoomPan {
        id: zoomPan
        anchors.fill: parent
        scene: scene
        area: mouseArea
    }

    MouseArea {
        id: mouseArea
        anchors.fill: parent
        enabled: true
        hoverEnabled: true
        acceptedButtons: Qt.AllButtons
    }

    Keys.onPressed: {
        if (event.key == Qt.Key_Up) {
            StackFrame.pop();
        } else if (event.key == Qt.Key_Down) {
            StackFrame.push();
        }
    }

    Component.onCompleted: updateSize();
    onWidthChanged: updateSize();
    onHeightChanged: updateSize();

    function updateSize() {
        image.x = (root.width - scene.imageSize.width) / 2;
        image.y = (root.height - scene.imageSize.height) / 2;
        image.width = scene.imageSize.width;
        image.height = scene.imageSize.height;
    }
}
